@@ -89,10 +89,10 @@ from IPython.core.display import Image, display
 import pickle
 
 ##### Suppress all TF2 and TF1.x warnings ###################
-try:
-    tf.logging.set_verbosity(tf.logging.ERROR)
-except:
-    tf.compat.v1.logging.set_verbosity(tf.compat.v1.logging.ERROR)
+tf2logger = tf.get_logger()
+tf2logger.warning('Silencing TF2.x warnings')
+tf2logger.root.removeHandler(tf2logger.root.handlers)
+tf.compat.v1.logging.set_verbosity(tf.compat.v1.logging.ERROR)
 ############################################################################################
 from tensorflow.keras.layers import Reshape, MaxPooling1D, MaxPooling2D, AveragePooling2D, AveragePooling1D
 from tensorflow.keras import Model, Sequential
@@ -471,10 +471,6 @@ def classify_columns(df_preds, model_options={}, verbose=0):
             print(' Missing columns = %s' %left_subtract(list(train),flat_list))
     return sum_all_cols
 #################################################################################
-from collections import defaultdict
-def nested_dictionary():
-    return defaultdict(nested_dictionary)
-#########################################################################################
 import re
 WORD = re.compile(r'\w+')
 def tokenize_fast(text):
@@ -926,6 +922,10 @@ def find_latitude_keyword(lat_word, columns, sel_columns=[]):
             if lat_word not in sel_columns:
                 sel_columns.append(lat_word)
             lat_keywords[lat_word] = 'latitude'
+        elif 'latitude' in lat_word.lower():
+            if lat_word not in sel_columns:
+                sel_columns.append(lat_word)
+            lat_keywords[lat_word] = 'latitude'
         elif 'lat' in lat_word.lower().split(" "):
             if lat_word not in sel_columns:
                 sel_columns.append(lat_word)
@@ -942,6 +942,10 @@ def find_latitude_keyword(lat_word, columns, sel_columns=[]):
             if lat_word not in sel_columns:
                 sel_columns.append(lat_word)
             lat_keywords[lat_word] = 'lat'
+        elif 'lat' in lat_word.lower():
+            if lat_word not in sel_columns:
+                sel_columns.append(lat_word)
+            lat_keywords[lat_word] = 'lat'
     return lat_keywords[lat_word]
 
 def find_longitude_keyword(lon_word, columns, sel_columns=[]):
@@ -972,6 +976,10 @@ def find_longitude_keyword(lon_word, columns, sel_columns=[]):
             if lon_word not in sel_columns:
                 sel_columns.append(lon_word)
             lon_keywords[lon_word] = 'longitude'
+        elif 'longitude' in lon_word.lower():
+            if lon_word not in sel_columns:
+                sel_columns.append(lon_word)
+            lon_keywords[lon_word] = 'longitude'
         elif 'lon' in lon_word.lower().split(" "):
             if lon_word not in sel_columns:
                 sel_columns.append(lon_word)
@@ -988,6 +996,10 @@ def find_longitude_keyword(lon_word, columns, sel_columns=[]):
             if lon_word not in sel_columns:
                 sel_columns.append(lon_word)
             lon_keywords[lon_word] = 'lon'
+        elif 'lon' in lon_word.lower():
+            if lon_word not in sel_columns:
+                sel_columns.append(lon_word)
+            lon_keywords[lon_word] = 'lon'
     return lon_keywords[lon_word]
 
 def find_longitude_columns(df, verbose=0):
@@ -1048,8 +1060,10 @@ from collections import defaultdict
 def nested_dictionary():
     return defaultdict(nested_dictionary)
 ############################################################################################
-def classify_dtypes_using_TF2(data_sample, idcols, verbose=0):
+def classify_dtypes_using_TF2(data_sample, preds, idcols, verbose=0):
     """
+    #### This works only on train data sets since they have both features and labels. ################
+    ####  It also works in only certain types of tf.data.datasets since every TF dataset is different format.
     If you send in a batch of Ttf.data.dataset with the name of target variable(s), you will get back
     all the features classified by type such as cats, ints, floats and nlps. This is all done using TF2.
     """
@@ -1061,7 +1075,6 @@ def classify_dtypes_using_TF2(data_sample, idcols, verbose=0):
     cats = []
     int_vocab = 0
     feats_max_min = nested_dictionary()
-
     #### Take(1) always displays only one batch only if num_epochs is set to 1 or a number. Otherwise No print! ########
     #### If you execute the below code without take, then it will go into an infinite loop if num_epochs was set to None.
     if data_sample.element_spec[0][preds[0]].shape[0] is None:
